/************************************************************************
 *   IRC - Internet Relay Chat, contrib/m_userlist.c
 *   Copyright (C) 2003 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id$
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "sprintf_irc.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#include <regex.h>
//#define REGEXEC_FLAGS	(REG_EXTENDED|REG_NOSUB)
#define REGEXEC_FLAGS 0

static void m_userlist(struct Client *client_p, struct Client *source_p,
                       int parc, char *parv[]);

struct Message userlist_msgtab = {
  "USERLIST", 0, 0, 1, 3, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, m_userlist}

};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&userlist_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&userlist_msgtab);
}

const char *_version = "$Revision$";
#endif

static void m_userlist(struct Client *client_p, struct Client *source_p,
                       int parc, char *parv[])
{
  char errbuf[1024];
  /* nick!user@host|gecos\0 */
  char nuhg[NICKLEN+1+USERLEN+1+HOSTLEN+1+REALLEN+1];
  int i, numfound=0;
  regex_t reg;
  regmatch_t m[1];
  struct Client *ptr;
  dlink_node *lp;

  if (parc == 3 && strcasecmp(parv[1], "-r"))
  {
    sendto_one(source_p,
               ":%s NOTICE %s :Usage: USERLIST [-r] <[nick!user@host|gecos]|[regex nick!user@host|gecos]>",
               me.name, source_p->name);
     return;
  }

  if (parc == 3)
  {
    if ((i=regcomp((regex_t *)&reg, parv[2], 1)))
    {
      regerror(i, (regex_t *)&reg, errbuf, sizeof(errbuf));
      sendto_one(source_p, ":%s NOTICE %s :regex error: %s",
                 me.name, source_p->name, errbuf);
      return;
    }

    DLINK_FOREACH(lp, global_client_list.head)
    {
      ptr = lp->data;

      if (!IsClient(ptr))
        continue;

      ircsprintf(nuhg, "%s!%s@%s|%s",
                 ptr->name, ptr->username, ptr->host, ptr->info);

      if (!regexec((regex_t *)&reg, nuhg, 1, m, REGEXEC_FLAGS))
      {
        if (!numfound++)
          sendto_one(source_p, ":%s NOTICE %s :Global matches for %s:",
                     me.name, source_p->name, parv[2]);

        sendto_one(source_p, ":%s NOTICE %s :  %s (%s@%s) [%s]",
                   me.name, source_p->name,
                   ptr->name, ptr->username, ptr->host, ptr->info);
      }
    }
  }
  else if (parc == 2)
  {
    DLINK_FOREACH(lp, global_client_list.head)
    {
      ptr = lp->data;

      if (!IsClient(ptr))
        continue;

      ircsprintf(nuhg, "%s!%s@%s|%s",
                 ptr->name, ptr->username, ptr->host, ptr->info);

      if (match(parv[1], nuhg))
      {
        if (!numfound++)
          sendto_one(source_p, ":%s NOTICE %s :Global matches for %s:",
                     me.name, source_p->name, parv[1]);

        sendto_one(source_p, ":%s NOTICE %s :  %s (%s@%s) [%s]",
                   me.name, source_p->name,
                   ptr->name, ptr->username, ptr->host, ptr->info);
      }
    }
  }

  if (numfound)
    sendto_one(source_p, ":%s NOTICE %s :End of list (%d match%s found)",
               me.name, source_p->name, numfound,
               (numfound == 1) ? "" : "es");
  else
    sendto_one(source_p, ":%s NOTICE %s :No matches found",
               me.name, source_p->name);

  regfree((regex_t *)&reg);
}
